<?php

//--------------------------------------------------

namespace Bbg\ProductAssets\Cache;

//--------------------------------------------------

//--------------------------------------------------

class RedisCache extends AbstractCache {

//--------------------------------------------------
// Public:Propertys


//--------------------------------------------------
// Private:Propertys
	private $redis;
	private $connected = false;

//--------------------------------------------------
// Protected:Propertys

	
//--------------------------------------------------
// Constants


//--------------------------------------------------
/**
* No Information Found
* @since 1.0
*/
	public function __construct(int $ttl=0, array $config=array()){
		$config = array_merge(array(
			"host" => "127.0.0.1", 
			"port" => 6379, 
			"data" => null, 
			"user" => null, 
			"pass" => null, 
			), $config);
		
		parent::__construct($ttl, $config);

		return $this->connect();
		}

//--------------------------------------------------
/**
* No Information Found
* @since 1.0
*/
	public function __destruct(){
		try{
			if($this->connected){
				$this->connected = false;
				$this->redis->close();
				}
			}
		catch(\RedisException $e){
			}
		}

//--------------------------------------------------
/**
* No Information Found
* @since 1.0
*/
	public function connect():bool {
		if(class_exists("\Redis")){
			try{
				if($this->redis = new \Redis()){
					if($this->redis->connect($this->config["host"], (int) $this->config["port"])){
						if(!empty($this->config["pass"])){
							$authData = array(
								"pass" => $this->config["pass"], 
								);
							
							if(!empty($this->config["user"])){
								$authData = array_merge($authData, array(
									"user" => $this->config["user"], 
									));
								}
							
							$this->redis->auth($authData);
							}
						
						$this->connected = true;
						}
					}
				}
			catch(\RedisException $e){
				}
			}
		
		return $this->connected;
		}

//--------------------------------------------------
/**
* No Information Found
* @since 1.0
*/
	public function exists(string $path, ?string $environment=null):bool {
		$key = $this->key($path, $environment);
		
		return $this->redis->exists($key);
		}

//--------------------------------------------------
/**
* No Information Found
* @since 1.0
*/
	public function read(string $path, ?string $environment=null):\stdClass {
		$key = $this->key($path, $environment);
		
		try{
			if($this->connected){
				if($this->redis->select((int) $this->config["data"])){
					if($this->redis->exists($key)){
						return json_decode($this->redis->get($key));
						}
					}
				}
			}
		catch(\RedisException $e){
			}
		
		return new \stdClass();
		}

//--------------------------------------------------
/**
* No Information Found
* @since 1.0
*/
	public function write(string $path, \stdClass $content, ?string $environment=null):bool {
        $key = $this->key($path, $environment);

		try{
			if($this->connected){
				if($this->redis->select((int) $this->config["data"])){
					$this->redis->set($key, json_encode($content));
					$this->redis->expire($key, (time() + $this->getTTL()));
					
					return true;
					}
				}
			}
		catch(\RedisException $e){
			}
		
		return false;
		}

//--------------------------------------------------
/**
* No Information Found
* @since 1.0
*/
	private function getFile(string $key):string {
		return $this->redis->getDbNum();
		}

//--------------------------------------------------

	}

//--------------------------------------------------

?>