<?php

//--------------------------------------------------

namespace Bbg\ProductAssets;

//--------------------------------------------------

use CurlHandle;

class HttpClient {

//--------------------------------------------------
// Public:Propertys


//--------------------------------------------------
// Private:Propertys
	private CurlHandle $curl;
	private array $httpheader;
	private int $error;
	private int $statuscode;

//--------------------------------------------------
// Protected:Propertys

	
//--------------------------------------------------
// Constants


//--------------------------------------------------
/**
* No Information Found
* @since 1.0
*/
	public function __construct(){
		$this->httpheader 		= array();
		$this->error 			= 0;
		$this->statuscode 		= 0;
		}

//--------------------------------------------------
/**
* No Information Found
* @since 7.0
*/
	public function getStatus(){
		return $this->statuscode;
		}

//--------------------------------------------------
/**
* No Information Found
* @since 7.0
*/
	public function getError(){
		return $this->error;
		}

//--------------------------------------------------
/**
* No Information Found
* @since 7.0
*/
	public function header(string $name, string $value):HttpClient {
		array_push($this->httpheader, $name.": ".$value);
		
		return $this;
		}

//--------------------------------------------------
/**
* No Information Found
* @since 7.0
*/
	public function request(string $method, string $url, ?array $data=null, ?array $options=null, bool $json=false){
		$options = $this->_options($options);
		
		if($json){
			$this->header("Content-Type", "application/json");
			}

		if($this->curl = curl_init()){
			if((is_array($options)) && (count($options) > 0)){
				foreach($options as $index => $option){
					if(is_array($option)){
						curl_setopt($this->curl, $option[0], $option[1]);
						}
					}
				}
			
			curl_setopt($this->curl, CURLOPT_CUSTOMREQUEST, $method);
			
			if(in_array($method, array("POST", "PUT", "PATCH"))){
				if($json){
					curl_setopt($this->curl, CURLOPT_POSTFIELDS, json_encode($data, JSON_UNESCAPED_UNICODE));
					}
				else{
					curl_setopt($this->curl, CURLOPT_POSTFIELDS, $data);
					}
				}
			else{
				$url = $this->appendQuery($url, $data);
				}
			
			curl_setopt($this->curl, CURLOPT_HTTPHEADER, $this->httpheader);
			curl_setopt($this->curl, CURLOPT_URL, $url);			

			$temporary = curl_exec($this->curl);

			if(($this->error = (int) curl_errno($this->curl)) != 0){
				$this->statuscode = (int) curl_getinfo($this->curl, CURLINFO_HTTP_CODE);
				
				return false;
				}
			
			$this->statuscode = curl_getinfo($this->curl, CURLINFO_HTTP_CODE);
			
			curl_close($this->curl);
			
			return $temporary;
			}
		
		return false;
		}

//--------------------------------------------------
/**
* No Information Found
* @since 7.0
*/
	private function appendQuery(string $url, ?array $data=null):string {
		if((is_array($data)) && (count($data) > 0)){
			$query = parse_url($url, PHP_URL_QUERY);
			
			if(!empty($query)){
				parse_str($query, $arguments);
				
				if((is_array($arguments)) && (count($arguments) > 0)){
					$data = array_merge($arguments, $data);
					}
				
				$url = strtok($url, "?");
				}
			
			$url .= "?".http_build_query($data, NULL, "&", PHP_QUERY_RFC3986);
			}
		
		return $url;
		}

//--------------------------------------------------
/**
* No Information Found
* @since 7.0
*/
	private function _options(?array $options):array {
		$options = (is_array($options) ? $options : array());
		$options = array_merge(array(
			array(CURLOPT_CONNECTTIMEOUT, 10), 
			array(CURLOPT_SSL_VERIFYHOST, 2), 
			array(CURLOPT_SSL_VERIFYPEER, true), 
			array(CURLOPT_RETURNTRANSFER, true), 
			), $options);
		
		return $options;
		}

//--------------------------------------------------

	}

//--------------------------------------------------

?>